import os
import sys

import pytest

from crashlog.models import Error


def pytest_configure(config):

    import warnings
    warnings.simplefilter('once', DeprecationWarning)
    HERE = os.path.join(os.path.dirname(__file__))
    sys.path.insert(0, os.path.join(HERE, 'demo'))


@pytest.fixture()
def error(db):
    return Error.objects.create(class_name='PermissionDenied',
                                message='Error Message',
                                traceback='',
                                url='/',
                                server_name='',
                                )



from django.contrib import messages
from django.db import models
from django.db.models.query import QuerySet
from django.urls import reverse
from django.utils.encoding import smart_str
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _


class ErrorManager(models.Manager):
    def get_queryset(self):
        return QuerySet(self.model, using=self._db).defer('message', 'traceback')

    get_query_set = get_queryset


class Error(models.Model):
    class_name = models.CharField(_('type'), max_length=128)
    message = models.TextField()
    traceback = models.TextField()
    date_time = models.DateTimeField(auto_now_add=True)
    url = models.URLField(null=True, blank=True)
    server_name = models.CharField(max_length=128, db_index=True)
    username = models.CharField(max_length=100, db_index=True, null=True, blank=True)
    objects = ErrorManager()

    def __unicode__(self):
        return "{0.date_time}: {0.class_name}".format(self)

    def __repr__(self):
        try:
            u = str(self)
        except (UnicodeEncodeError, UnicodeDecodeError):
            u = '[Bad Unicode data]'
        return smart_str(u'<Error: %s>' % u)

    def get_change_url(self):
        return reverse('admin:crashlog_error_change', args=[self.pk])

    def message_user(self, request, message, level=messages.ERROR, extra_tags='', fail_silently=False,
                     detail_link=True):
        if detail_link:
            message = mark_safe("{} (<a href='{}'>details</a>)".format(message, self.get_change_url()))

        messages.add_message(request, level, message, extra_tags=extra_tags,
                             fail_silently=fail_silently)

    class Meta:
        app_label = 'crashlog'



# import adminactions.urls
from django.contrib.admin import autodiscover, site
from django.contrib.auth.decorators import login_required
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.http import HttpResponse
from django.urls import path

from crashlog.middleware import process_exception

autodiscover()


def raise_error(request):
    raise IndexError('IndexError')


@login_required
def raise_with_message_user(request):
    try:
        raise IndexError('IndexError')
    except Exception as e:
        process_exception(e, request, message_user=True)
    return HttpResponse()


urlpatterns = [path('/raise-error', raise_error, name='raise-error'),
               path('/raise_with_message_user', raise_with_message_user, name='raise_with_message_user'),
               path('', site.urls),
               ] + staticfiles_urlpatterns()



from admin_extra_urls.api import button
from admin_extra_urls.mixins import ExtraUrlMixin, _confirm_action
from django.contrib import admin
from django.db import connection

from crashlog.models import Error


class ErrorAdmin(ExtraUrlMixin, admin.ModelAdmin):
    change_form_template = "admin/crashlog/error_form.html"
    date_hierarchy = 'date_time'
    list_display = ('class_name', 'message', 'date_time', 'url', 'username')
    list_filter = ('class_name', 'date_time', 'server_name')
    search_fields = ('username', 'class_name', 'message', 'url')
    ordering = ('-date_time',)

    @button(label='Empty Log', css_class="btn btn-danger", icon="icon-trash icon-white")
    def empty_log(self, request):
        def _action(request):
            cursor = connection.cursor()
            cursor.execute('TRUNCATE TABLE "{0}"'.format(Error._meta.db_table))

        return _confirm_action(self, request, _action,
                               "Confirm deletion whole error log",
                               "Successfully executed")

    def has_add_permission(self, request):
        return False


admin.site.register(Error, ErrorAdmin)



from django.conf import settings
from django.core.signals import setting_changed
from django.utils.module_loading import import_string

IGNORE_EXCEPTIONS = []


class Config:
    _ignore_exceptions = []


def update_config(sender=None, setting=None, value=None, enter=False, **kwargs):
    conf._ignore_exceptions = [import_string(a) for a in (getattr(settings, 'CRASHLOG_IGNORE_EXCEPTIONS',
                                      IGNORE_EXCEPTIONS))]


conf = Config()
update_config()

setting_changed.connect(update_config)
